@@ -7,10 +7,11 @@ from importlib import import_module
 from os.path import join, dirname, abspath, exists, splitext
 
 import scrapy
-from scrapy.command import ScrapyCommand
+from scrapy.commands import ScrapyCommand
 from scrapy.utils.template import render_templatefile, string_camelcase
 from scrapy.exceptions import UsageError
 
+
 def sanitize_module_name(module_name):
     """Sanitize the given module name, by replacing dashes and points
     with underscores and prefixing it with a letter if it doesn't start
@@ -21,6 +22,7 @@ def sanitize_module_name(module_name):
         module_name = "a" + module_name
     return module_name
 
+
 class Command(ScrapyCommand):
 
     requires_project = True
@@ -40,7 +42,7 @@ class Command(ScrapyCommand):
             help="Edit spider after creating it")
         parser.add_option("-d", "--dump", dest="dump", metavar="TEMPLATE",
             help="Dump template to standard output")
-        parser.add_option("-t", "--template", dest="template", default="crawl",
+        parser.add_option("-t", "--template", dest="template", default="basic",
             help="Uses a custom template.")
         parser.add_option("--force", dest="force", action="store_true",
             help="If the spider already exists, overwrite it with the template")
@@ -52,7 +54,8 @@ class Command(ScrapyCommand):
         if opts.dump:
             template_file = self._find_template(opts.dump)
             if template_file:
-                print(open(template_file, 'r').read())
+                with open(template_file, "r") as f:
+                    print(f.read())
             return
         if len(args) != 2:
             raise UsageError()
@@ -65,15 +68,14 @@ class Command(ScrapyCommand):
             return
 
         try:
-            crawler = self.crawler_process.create_crawler()
-            spider = crawler.spiders.create(name)
+            spidercls = self.crawler_process.spider_loader.load(name)
         except KeyError:
             pass
         else:
             # if spider already exists and not --force then halt
             if not opts.force:
                 print("Spider %r already exists in module:" % name)
-                print("  %s" % spider.__module__)
+                print("  %s" % spidercls.__module__)
                 return
         template_file = self._find_template(opts.template)
         if template_file:
@@ -89,8 +91,8 @@ class Command(ScrapyCommand):
             'module': module,
             'name': name,
             'domain': domain,
-            'classname': '%sSpider' % ''.join([s.capitalize() \
-                for s in module.split('_')])
+            'classname': '%sSpider' % ''.join(s.capitalize() \
+                for s in module.split('_'))
         }
         spiders_module = import_module(self.settings['NEWSPIDER_MODULE'])
         spiders_dir = abspath(dirname(spiders_module.__file__))
